package com.jz.dmp.modules.controller.DataIntegration;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.utils.realTime.CmdUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.model.DmpRealtimeSyncInfo;
import com.jz.dmp.modules.model.RealTimeSyncModel;
import com.jz.dmp.modules.service.DmpRealtimeSyncInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 实时同步任务
 *
 * @author Bellamy
 * @since 2021-01-05 10:56:18
 */
@RestController
@RequestMapping("/realTimeSync")
@Api(tags = "数据集成--实时同步任务")
public class RealTimeSyncController {

    private static Logger logger = LoggerFactory.getLogger(RealTimeSyncController.class);
    /**
     * 服务对象
     */
    @Autowired
    private DmpRealtimeSyncInfoService dmpRealtimeSyncInfoService;

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "实时同步任务列表分页查询", notes = "实时同步任务列表分页查询")
    @PostMapping(value = "/realTimeSyncListPage")
    public PageInfoResponse<RealTimeSyncListDto> getDataSourceListPage(@RequestBody @Validated RealTimeSyncListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<RealTimeSyncListDto> pageInfo = new PageInfoResponse<RealTimeSyncListDto>();
        if (StringUtils.isEmpty(req.getProjectId())) {
            pageInfo.setMessage("项目id不能为空！");
            pageInfo.setCode(ResultCode.PARAMS_ERROR);
            return pageInfo;
        }
        try {
            pageInfo = dmpRealtimeSyncInfoService.queryRealTimeSyncListPage(req);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 批量启动/停止启动实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "批量启动/停止运行实时同步任务", notes = "批量启动实时同步任务")
    @GetMapping(value = "/startRealTimeSync")
    @ApiImplicitParams({@ApiImplicitParam(name = "taskId", value = "任务id", required = true),
            @ApiImplicitParam(name = "type", value = "01:运行，02：停止", required = true),
            @ApiImplicitParam(name = "projectId", value = "项目id", required = true)})
    public JsonResult startRealTimeSync(@RequestParam String taskId, @RequestParam String projectId, @RequestParam String type) throws Exception {
        if (StringUtils.isEmpty(taskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        if (StringUtils.isEmpty(type)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "type不能为空！");
        }
        JsonResult result = new JsonResult();
        try {
            result = dmpRealtimeSyncInfoService.executeRealtimeTask(taskId, type, projectId);
        } catch (Exception e) {
            e.printStackTrace();
            result.setMessage("执行失败！");
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
        }
        return result;
    }

    /**
     * 删除实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "删除实时同步任务", notes = "删除实时同步任务")
    @GetMapping(value = "/delRealTimeSync")
    @ApiImplicitParam(name = "realTaskId", value = "任务id")
    public JsonResult delRealTimeSync(@RequestParam String realTaskId) throws Exception {
        logger.info("###################请求参数{}taskId=" + realTaskId + "###################");
        if (StringUtils.isEmpty(realTaskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        return dmpRealtimeSyncInfoService.deleteByrealTaskId(realTaskId);
    }

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @ApiOperation(value = "获取数据源和目标数据源下拉框", notes = "获取数据源和目标数据源下拉框")
    @GetMapping(value = "/getDatasourceNameList")
    @ApiImplicitParams({@ApiImplicitParam(name = "projectId", value = "项目id", required = true),
            @ApiImplicitParam(name = "type", value = "数据源类型：01来源，02 目标源", required = true)})
    public JsonResult<List<DataSourceNameListDto>> getDatasourceNameList(@RequestParam String projectId, @RequestParam String type) throws Exception {
        logger.info("###################请求参数{}projectId=" + projectId + "&type=" + type);
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(type)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "类型不能为空！");
        }
        JsonResult<List<DataSourceNameListDto>> jsonResult = dmpRealtimeSyncInfoService.queryDatasourceNameList(projectId, type);
        return jsonResult;
    }

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @ApiOperation(value = "新增--选择源表信息和配置目标表", notes = "新增--选择源表信息和配置目标表")
    @PostMapping(value = "/getSourceDbTableList")
    public JsonResult<RealTimeSyncModel> getTableInfo(@RequestBody @Validated RealTimeTableInfoReq req, HttpServletRequest httpRequest) throws Exception {
        logger.info("###################请求参数{}" + req.toString() + "############");
        if (StringUtils.isEmpty(req.getProjectId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(req.getSrcDatasourceId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源数据源id不能为空！");
        }
        JsonResult<RealTimeSyncModel> jsonResult = dmpRealtimeSyncInfoService.queryTableInfoByParams(req);
        return jsonResult;
    }

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @ApiOperation(value = "新增--选择目标信息", notes = "新增--选择目标信息")
    @GetMapping(value = "/targetDatasourceInfo")
    @ApiImplicitParam(name = "projectId", value = "项目id", required = true)
    public JsonResult getTargetDatasourceInfo(@RequestParam String projectId) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        JsonResult jsonResult = dmpRealtimeSyncInfoService.selectTargetDatasourceInfo(projectId);
        return jsonResult;
    }

    /**
     * 设置成黑名单时 ,检查是否有同步任务存在运行
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @ApiOperation(value = "设置成黑名单时 ,检查是否有同步任务存在运行", notes = "设置成黑名单时 ,检查是否有同步任务存在运行")
    @GetMapping(value = "/settingBlackListCheckTask")
    @ApiImplicitParams({@ApiImplicitParam(name = "srcDatasourceId", value = "来源数据源id", required = true),
            @ApiImplicitParam(name = "sourceTableName", value = "来源表名称", required = true)})
    public JsonResult settingBlackListCheckTask(@RequestParam String srcDatasourceId, @RequestParam String sourceTableName) throws Exception {
        logger.info("###################请求参数{}srcDatasourceId=" + srcDatasourceId + "&sourceTableName=" + sourceTableName);
        if (StringUtils.isEmpty(srcDatasourceId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源数据源id不能为空！");
        }
        if (StringUtils.isEmpty(sourceTableName)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源表名称不能为空！");
        }
        JsonResult jsonResult = dmpRealtimeSyncInfoService.queryRealTimeInfoByDataSourceId(srcDatasourceId, sourceTableName);
        return jsonResult;
    }

    /**
     * 保存/编辑实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    @ApiOperation(value = "保存/编辑实时同步任务", notes = "保存实时同步任务")
    @PostMapping(value = "/addTask")
    public JsonResult addTask(@RequestBody Map<String, Object> params, HttpServletRequest httpRequest) throws Exception {
        logger.info("###################请求参数{}" + params.toString() + "############");
        if (StringUtils.isEmpty(params.get("projectId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("srcDataSourceId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源数据源id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("targetDataSourceId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "目标数据源id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("treeId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "treeId不能为空！");
        }

        JsonResult result = new JsonResult();
        try {
            result = dmpRealtimeSyncInfoService.addRealTimeTask(params);
        } catch (Exception e) {
            result.setMessage(e.getMessage());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 编辑实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
  /*  @ApiOperation(value = "编辑实时同步任务", notes = "编辑实时同步任务")
    @PostMapping(value = "/updateTask")
    public JsonResult updateTask(@RequestBody Map<String, Object> params, HttpServletRequest httpRequest) throws Exception {
        logger.info("################请求参数{}" + params.toString() + "############");
        if (StringUtils.isEmpty(params.get("projectId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "项目id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("srcDataSourceId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "来源数据源id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("targetDataSourceId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "目标数据源id不能为空！");
        }
        if (StringUtils.isEmpty(params.get("taskId").toString())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }

        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpRealtimeSyncInfoService.updateRealTimeTask(params);
        } catch (Exception e) {
            e.printStackTrace();
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
        }
        return jsonResult;
    }*/

    /**
     * 编辑--数据回显，通过id查询任务相关信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-12
     */
    @ApiOperation(value = "编辑--数据回显", notes = "编辑--数据回显")
    @GetMapping(value = "/editDataEcho")
    @ApiImplicitParam(name = "taskId", value = "实时任务id", required = true)
    public JsonResult<RealTimeEditDataEchoDto> getRealtimeTaskInfo(@RequestParam String taskId) throws Exception {
        logger.info("###################请求参数{}taskId=" + taskId + "###################");
        if (StringUtils.isEmpty(taskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "实时任务id不能为空！");
        }
        JsonResult<RealTimeEditDataEchoDto> jsonResult = dmpRealtimeSyncInfoService.selectRealtimeTaskById(taskId);
        return jsonResult;
    }

    /**
     * 批量上下线
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "批量上下线", notes = "批量上下线")
    @GetMapping(value = "/batchUptOnlineStatus")
    @ApiImplicitParams({@ApiImplicitParam(name = "realTaskId", value = "任务id", required = true),
            @ApiImplicitParam(name = "onlineStatus", value = "上下线状态:Y 上线，N 下线", required = true)})
    public JsonResult batchUptOnlineStatus(@RequestParam String realTaskId, @RequestParam String onlineStatus) throws Exception {
        logger.info("###################请求参数{}taskId=" + realTaskId + "###################");
        if (StringUtils.isEmpty(realTaskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        if (StringUtils.isEmpty(onlineStatus)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "状态不能为空！");
        }
        boolean jsonResult = dmpRealtimeSyncInfoService.batchUptOnlineStatus(realTaskId, onlineStatus);
        if (jsonResult) {
            return JsonResult.ok();
        } else {
            return JsonResult.error("删除失败！");
        }
    }

    /**
     * 冲突检查
     *
     * @return
     * @author Bellamy
     * @since 2021-02-22
     */
    @ApiOperation(value = "冲突检查", notes = "冲突检查")
    @PostMapping(value = "/conflictCheck")
    public JsonResult conflictCheck(@RequestBody ConflictCheckReq params, HttpServletRequest httpRequest) throws Exception {
        logger.info("###################请求参数{}" + params.toString() + "############");
        if (StringUtils.isEmpty(params.getTargetDataSourceId())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "目标数据源id不能空！");
        }
        if (StringUtils.isEmpty(params.getTablesName())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "表名称不能为空！");
        }

        JsonResult result = new JsonResult();
        try {
            result = dmpRealtimeSyncInfoService.conflictCheck(params);
        } catch (Exception e) {
            result.setMessage(e.getMessage());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }

}